import { EventEmitter } from 'events';

/**
 * Configuration options for database initialization.
 *
 * @example
 * ```typescript
 * const options: DatabaseOptions = {
 *   path: './data/mydb',
 *   encrypt: true,
 *   encryptionKey: 'your-secret-key',
 *   cacheDocuments: true,
 *   maxCacheSize: 1000
 * };
 * ```
 */
interface DatabaseOptions {
    /** Storage path for database files. Defaults to './nubodb'. */
    path?: string;
    /** Store data in memory only (no persistence). Defaults to false. */
    inMemory?: boolean;
    /** Create database directory if it doesn't exist. Defaults to true. */
    createIfMissing?: boolean;
    /** Enable encryption for stored data. Defaults to false. */
    encrypt?: boolean;
    /** Encryption algorithm to use. Defaults to 'aes-256-cbc'. */
    encryptionMethod?: 'aes-256-cbc' | 'aes-256-gcm' | 'chacha20-poly1305';
    /** Encryption key for data security. Required if encrypt is true. */
    encryptionKey?: string;
    /** Key derivation function for encryption. Defaults to 'pbkdf2'. */
    encryptionKDF?: 'pbkdf2' | 'scrypt' | 'argon2';
    /** Automatically flush changes to disk. Defaults to true. */
    autoFlush?: boolean;
    /** Interval in milliseconds for automatic flushing. Defaults to 1000. */
    flushInterval?: number;
    /** Enable in-memory document caching. Defaults to true. */
    cacheDocuments?: boolean;
    /** Maximum number of documents to cache. Defaults to 1000. */
    maxCacheSize?: number;
    /** Enable automatic indexing. Defaults to true. */
    enableIndexing?: boolean;
    /** Schema validation mode. Defaults to 'warn'. */
    schemaValidation?: 'strict' | 'warn' | 'ignore';
    /** Path to schema definition file. */
    schemaPath?: string;
    /** Enable debug logging. Defaults to false. */
    debug?: boolean;
    /** Logging level for debug output. Defaults to 'info'. */
    logLevel?: 'error' | 'warn' | 'info' | 'debug';
}
/**
 * Definition of a single field in a document schema.
 *
 * @example
 * ```typescript
 * const userSchema: Schema = {
 *   name: {
 *     type: 'string',
 *     required: true,
 *     min: 2,
 *     max: 50
 *   },
 *   email: {
 *     type: 'string',
 *     required: true,
 *     unique: true,
 *     pattern: /^[^\s@]+@[^\s@]+\.[^\s@]+$/
 *   },
 *   age: {
 *     type: 'number',
 *     min: 0,
 *     max: 150
 *   }
 * };
 * ```
 */
interface SchemaField {
    /** Data type for this field. */
    type: 'string' | 'number' | 'boolean' | 'object' | 'array' | 'date' | 'buffer';
    /** Whether this field is required. Defaults to false. */
    required?: boolean;
    /** Default value when field is not provided. */
    default?: any;
    /** Whether this field must have unique values across documents. */
    unique?: boolean;
    /** Whether to create an index on this field for faster queries. */
    index?: boolean;
    /** Minimum value (for numbers) or length (for strings/arrays). */
    min?: number;
    /** Maximum value (for numbers) or length (for strings/arrays). */
    max?: number;
    /** Regular expression pattern for string validation. */
    pattern?: RegExp;
    /** Array of allowed values for this field. */
    enum?: any[];
    /** Reference to another collection (for relationships). */
    ref?: string;
    /** Custom validation function. Return true for valid, string for error message. */
    validate?: (value: any) => boolean | string;
}
/**
 * Complete schema definition for a collection.
 * Maps field names to their schema definitions.
 *
 * @example
 * ```typescript
 * const userSchema: Schema = {
 *   name: { type: 'string', required: true },
 *   email: { type: 'string', required: true, unique: true },
 *   age: { type: 'number', min: 0 },
 *   isActive: { type: 'boolean', default: true }
 * };
 * ```
 */
interface Schema {
    /** Schema field definitions indexed by field name. */
    [fieldName: string]: SchemaField;
}
/**
 * Base document interface with system metadata.
 * All documents stored in NuboDB extend this interface.
 *
 * @example
 * ```typescript
 * interface User extends Document {
 *   name: string;
 *   email: string;
 *   age?: number;
 * }
 * ```
 */
interface Document {
    /** Unique identifier for this document. */
    _id: string;
    /** Timestamp when document was created. */
    _createdAt: Date;
    /** Timestamp when document was last updated. */
    _updatedAt: Date;
    /** Document version for optimistic concurrency control. */
    _version?: number;
    /** Additional user-defined fields. */
    [key: string]: any;
}
/**
 * Lightweight document metadata without full content.
 * Used for efficient document listings and cache management.
 */
interface DocumentMetadata {
    /** Document unique identifier. */
    id: string;
    /** Creation timestamp. */
    createdAt: Date;
    /** Last update timestamp. */
    updatedAt: Date;
    /** Document size in bytes. */
    size: number;
    /** Document content checksum for integrity verification. */
    checksum?: string;
    /** Document version number. */
    version?: number;
}
/**
 * Supported query operators for filtering documents.
 * Similar to MongoDB query operators but with NuboDB-specific implementations.
 *
 * @example
 * ```typescript
 * // Equality and comparison
 * { age: { $gte: 18, $lt: 65 } }
 *
 * // Array membership
 * { status: { $in: ['active', 'pending'] } }
 *
 * // Logical operators
 * { $and: [{ age: { $gte: 18 } }, { status: 'active' }] }
 *
 * // Text search
 * { name: { $regex: /john/i } }
 * ```
 */
type QueryOperator = 
/** Equality operator. */
'$eq'
/** Not equal operator. */
 | '$ne'
/** Greater than operator. */
 | '$gt'
/** Greater than or equal operator. */
 | '$gte'
/** Less than operator. */
 | '$lt'
/** Less than or equal operator. */
 | '$lte'
/** Value exists in array operator. */
 | '$in'
/** Value not in array operator. */
 | '$nin'
/** Field exists operator. */
 | '$exists'
/** Regular expression match operator. */
 | '$regex'
/** Full-text search operator. */
 | '$text'
/** Logical AND operator. */
 | '$and'
/** Logical OR operator. */
 | '$or'
/** Logical NOR operator. */
 | '$nor'
/** Logical NOT operator. */
 | '$not';
/**
 * Query condition with operator and value.
 */
interface QueryCondition {
    [operator: string]: any;
}
/**
 * MongoDB-like query filter for finding documents.
 */
interface QueryFilter {
    [field: string]: any | QueryCondition;
}
/**
 * Options for query operations like sorting, pagination, and projection.
 */
interface QueryOptions {
    limit?: number;
    skip?: number;
    sort?: {
        [field: string]: 1 | -1;
    } | Array<[string, 1 | -1]>;
    projection?: {
        [field: string]: 0 | 1;
    };
    explain?: boolean;
    hint?: string;
    timeout?: number;
}
/**
 * Configuration options for collections.
 */
interface CollectionOptions {
    schema?: Schema;
    encrypt?: boolean;
    compression?: boolean;
    indexes?: IndexDefinition[];
    maxDocuments?: number;
    autoIndex?: boolean;
    maxCacheSize?: number;
}
/**
 * Definition of a database index.
 */
interface IndexDefinition {
    fields: {
        [field: string]: 1 | -1;
    };
    unique?: boolean;
    sparse?: boolean;
    name?: string;
}
/**
 * Result of a single document insert operation.
 */
interface InsertResult {
    id: string;
    success: boolean;
    document: Document;
    insertedCount: number;
}
/**
 * Result of a bulk document insert operation.
 */
interface InsertManyResult {
    insertedIds: string[];
    insertedCount: number;
    success: boolean;
}
/**
 * Result of a document update operation.
 */
interface UpdateResult {
    modifiedCount: number;
    success: boolean;
    upsertedId?: string;
    upsertedCount?: number;
}
/**
 * Result of a document delete operation.
 */
interface DeleteResult {
    deletedCount: number;
    success: boolean;
}
/**
 * Result of a document find operation with pagination metadata.
 */
interface FindResult<T = Document> {
    documents: T[];
    total: number;
    hasMore: boolean;
    page?: number;
    totalPages?: number;
}
/**
 * Fluent interface for building complex queries.
 */
interface QueryBuilder$1<T = Document> {
    where(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    and(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    or(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    sort(field: string, direction: 1 | -1): QueryBuilder$1<T>;
    limit(count: number): QueryBuilder$1<T>;
    skip(count: number): QueryBuilder$1<T>;
    select(fields: string[]): QueryBuilder$1<T>;
    execute(): Promise<FindResult<T>>;
    findOne(): Promise<T | null>;
    count(): Promise<number>;
}
/**
 * Database transaction interface (placeholder for future implementation).
 */
interface Transaction {
    commit(): Promise<void>;
    rollback(): Promise<void>;
    collection<T = Document>(name: string): any;
}
/**
 * Database-wide statistics and metrics.
 */
interface DatabaseStats {
    collections: number;
    totalDocuments: number;
    totalSize: number;
    indexes: number;
    uptime: number;
}
/**
 * Event types emitted by the database.
 */
interface DatabaseEvents {
    'document:inserted': (collection: string, document: Document) => void;
    'document:updated': (collection: string, document: Document) => void;
    'document:deleted': (collection: string, documentId: string) => void;
    'collection:created': (name: string) => void;
    'collection:dropped': (name: string) => void;
    error: (error: Error) => void;
}

/**
 * Thin wrapper around the Node.js `fs` module that provides collection-aware
 * persistence utilities. Documents are stored as individual JSON files inside
 * a directory named after the collection.
 */
declare class FileStorage {
    private basePath;
    private ensuredDirs;
    private readonly MAX_CONCURRENT_FILES;
    /**
     * @param basePath - Root directory where all collections will be stored.
     */
    constructor(basePath: string);
    /** Ensure a directory exists */
    ensureDirectory(path: string): Promise<void>;
    /**
     * Persist a single document.
     *
     * @param collectionPath - Collection folder relative to `basePath`.
     * @param document       - Document with metadata to write.
     */
    writeDocument(collectionPath: string, document: Document): Promise<void>;
    /**
     * Read a single document.
     *
     * @returns The document or `null` if the file doesn't exist.
     */
    readDocument(collectionPath: string, documentId: string): Promise<Document | null>;
    /**
     * Read every document stored in a collection directory.
     *
     * @returns An array of documents.
     */
    readAllDocuments(collectionPath: string): Promise<Document[]>;
    /**
     * Delete a single document file.
     *
     * @returns `true` if the document was deleted, `false` if it didn't exist.
     */
    deleteDocument(collectionPath: string, documentId: string): Promise<boolean>;
    /**
     * Check existence of a document without reading it.
     *
     * @returns `true` if the document exists, `false` if it doesn't.
     */
    documentExists(collectionPath: string, documentId: string): Promise<boolean>;
    /** Lightweight metadata fetch */
    getDocumentMetadata(collectionPath: string, documentId: string): Promise<DocumentMetadata | null>;
}

/**
 * Public facade that combines CRUD (DocumentOperations) and query (QueryOperations)
 * capabilities for a collection.
 *
 * @typeParam T - Shape of the documents stored in this collection.
 */
declare class Collection<T = Document> {
    private documentOps;
    private queryOps;
    /**
     * Create a new Collection instance.
     *
     * @param name     - Logical name of the collection (also directory name on disk).
     * @param storage  - Low-level storage engine implementation.
     * @param options  - Optional collection-specific configuration.
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /**
     * Insert a single document into the collection.
     *
     * @param data - Document data to insert. Missing fields will use schema defaults.
     * @returns Result containing the inserted document's ID and success status.
     * @throws {DocumentError} When document fails validation or insertion.
     * @example
     * ```typescript
     * const result = await collection.insert({ name: 'John', age: 30 });
     * console.log('Inserted with ID:', result.insertedId);
     * ```
     * @since 1.0.0
     */
    insert(data: Partial<T>): Promise<InsertResult>;
    /**
     * Insert multiple documents in a single operation.
     *
     * @param documents - Array of document data to insert.
     * @returns Result containing all inserted document IDs and success status.
     * @throws {DocumentError} When any document fails validation or insertion.
     * @example
     * ```typescript
     * const result = await collection.insertMany([
     *   { name: 'John', age: 30 },
     *   { name: 'Jane', age: 25 }
     * ]);
     * console.log('Inserted IDs:', result.insertedIds);
     * ```
     * @since 1.0.0
     */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /**
     * Update all documents that match the specified filter.
     *
     * @param filter - Query criteria to match documents for updating.
     * @param updateData - Partial document data to apply to matched documents.
     * @returns Result containing count of modified documents and success status.
     * @throws {DocumentError} When update operation fails or validation errors occur.
     * @example
     * ```typescript
     * const result = await collection.update(
     *   { status: 'pending' },
     *   { status: 'completed', updatedAt: new Date() }
     * );
     * console.log('Updated documents:', result.modifiedCount);
     * ```
     * @since 1.0.0
     */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Update existing documents or insert a new one if no matches found.
     *
     * @param filter - Query criteria to match documents for updating.
     * @param updateData - Document data to apply or insert if no matches.
     * @returns Result containing modified/inserted count and success status.
     * @throws {DocumentError} When upsert operation fails or validation errors occur.
     * @example
     * ```typescript
     * const result = await collection.upsert(
     *   { email: 'user@example.com' },
     *   { name: 'Updated Name', email: 'user@example.com' }
     * );
     * console.log('Upserted:', result.upsertedId || 'Updated existing');
     * ```
     * @since 1.0.0
     */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Delete all documents that match the specified filter.
     *
     * @param filter - Query criteria to match documents for deletion.
     * @returns Result containing count of deleted documents and success status.
     * @throws {DocumentError} When delete operation fails.
     * @example
     * ```typescript
     * const result = await collection.delete({ status: 'archived' });
     * console.log('Deleted documents:', result.deletedCount);
     * ```
     * @since 1.0.0
     */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Delete the first document that matches the filter.
     *
     * @param filter - Query criteria to match a single document for deletion.
     * @returns Result containing count of deleted documents (0 or 1) and success status.
     * @throws {DocumentError} When delete operation fails.
     * @example
     * ```typescript
     * const result = await collection.deleteOne({ _id: 'user123' });
     * if (result.deletedCount > 0) {
     *   console.log('Document deleted successfully');
     * }
     * ```
     * @since 1.0.0
     */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Manually create an index on the collection for improved query performance.
     *
     * @param definition - Index definition specifying fields and options.
     * @throws {DatabaseError} When index creation fails.
     * @example
     * ```typescript
     * await collection.createIndex({
     *   fields: ['email'],
     *   unique: true,
     *   name: 'email_unique_idx'
     * });
     * ```
     * @since 1.0.0
     */
    createIndex(definition: IndexDefinition): Promise<void>;
    /**
     * Find documents matching a filter.
     *
     * @param filter   - Query criteria (MongoDB-like).
     * @param options  - Pagination, sorting, projection …
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /**
     * Return the first document that matches the filter.
     *
     * @param filter - Query criteria to match documents. Defaults to empty object (match all).
     * @returns First matching document or null if no matches found.
     * @throws {DatabaseError} When query operation fails.
     * @example
     * ```typescript
     * const user = await collection.findOne({ email: 'user@example.com' });
     * if (user) {
     *   console.log('Found user:', user.name);
     * }
     * ```
     * @since 1.0.0
     */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /**
     * Lookup a document by its unique `_id` field.
     *
     * @param id - The document ID to search for.
     * @returns Document with matching ID or null if not found.
     * @throws {DatabaseError} When query operation fails.
     * @example
     * ```typescript
     * const user = await collection.findById('user123');
     * if (user) {
     *   console.log('Found user:', user);
     * }
     * ```
     * @since 1.0.0
     */
    findById(id: string): Promise<T | null>;
    /**
     * Start a fluent QueryBuilder chain for complex queries.
     *
     * @returns QueryBuilder instance for chaining query operations.
     * @example
     * ```typescript
     * const results = await collection
     *   .query()
     *   .where('age').gte(18)
     *   .and('status').equals('active')
     *   .sort({ name: 1 })
     *   .limit(10)
     *   .execute();
     * ```
     * @since 1.0.0
     */
    query(): QueryBuilder$1<T>;
    /**
     * Count documents that match the specified filter.
     *
     * @param filter - Query criteria to match documents. Defaults to empty object (count all).
     * @returns Number of documents matching the filter.
     * @throws {DatabaseError} When count operation fails.
     * @example
     * ```typescript
     * const totalUsers = await collection.count();
     * const activeUsers = await collection.count({ status: 'active' });
     * console.log(`${activeUsers} out of ${totalUsers} users are active`);
     * ```
     * @since 1.0.0
     */
    count(filter?: QueryFilter): Promise<number>;
    /**
     * Check if the collection currently has zero documents.
     *
     * @returns True if collection is empty, false otherwise.
     * @throws {DatabaseError} When count operation fails.
     * @example
     * ```typescript
     * if (await collection.isEmpty()) {
     *   console.log('Collection is empty, initializing default data...');
     * }
     * ```
     * @since 1.0.0
     */
    isEmpty(): Promise<boolean>;
    /**
     * Clear the in-memory LRU cache for this collection to free memory.
     *
     * @example
     * ```typescript
     * collection.clearCache(); // Free up memory
     * ```
     * @since 1.0.0
     */
    clearCache(): void;
    /**
     * Return collection statistics including size, document count, and cache info.
     *
     * @returns Object containing collection metrics and performance data.
     * @throws {DatabaseError} When stats operation fails.
     * @example
     * ```typescript
     * const stats = await collection.stats();
     * console.log(`Collection has ${stats.totalDocuments} documents (${stats.totalSize} bytes)`);
     * ```
     * @since 1.0.0
     */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
    /**
     * Initialize the collection by setting up document and query operations.
     * Lazily called by BaseCollection; can also be invoked manually.
     *
     * @throws {DatabaseError} When initialization fails.
     * @example
     * ```typescript
     * await collection.initialize(); // Ensure collection is ready
     * ```
     * @since 1.0.0
     */
    initialize(): Promise<void>;
}

/**
 * Main database instance that manages collections and provides the public API.
 * Implements the singleton pattern for global database access.
 */
declare class NuboDB extends EventEmitter {
    private static instance;
    private options;
    private storage;
    private encryptionManager?;
    private collections;
    private isOpen;
    private startTime;
    private logger;
    /**
     * Private constructor for singleton pattern.
     *
     * @param options - Database configuration including storage path, encryption, etc.
     */
    private constructor();
    /**
     * Get the singleton database instance. Creates one if it doesn't exist.
     *
     * @param options - Required for first initialization.
     */
    static getInstance(options?: DatabaseOptions): Promise<NuboDB>;
    /**
     * Create a new database instance (bypasses singleton).
     *
     * @param options - Database configuration.
     */
    static create(options: DatabaseOptions): Promise<NuboDB>;
    /**
     * Initialize the database and create storage directories.
     *
     * @throws {DatabaseError} When database fails to open or is already open.
     * @example
     * ```typescript
     * const db = await NuboDB.create({ path: './mydb' });
     * await db.open();
     * ```
     * @since 1.0.0
     */
    open(): Promise<void>;
    /**
     * Clean up resources and close all collections.
     *
     * @throws {DatabaseError} When database fails to close properly.
     * @example
     * ```typescript
     * await db.close();
     * ```
     * @since 1.0.0
     */
    close(): Promise<void>;
    /**
     * Get or create a collection by name.
     *
     * @param name    - Collection name.
     * @param options - Optional collection-specific settings.
     * @returns Collection instance for CRUD operations.
     */
    collection<T = any>(name: string, options?: CollectionOptions): Collection<T>;
    /**
     * Explicitly create a new collection with optional schema.
     *
     * @param name    - Collection name.
     * @param schema  - Optional schema for validation.
     * @param options - Collection configuration.
     */
    createCollection<T = any>(name: string, schema?: Schema, options?: CollectionOptions): Promise<Collection<T>>;
    /**
     * Delete all documents in a collection and remove it from memory.
     *
     * @param name - Collection to drop.
     * @returns Success status.
     */
    dropCollection(name: string): Promise<boolean>;
    /**
     * Get names of all currently loaded collections.
     *
     * @returns Array of collection names currently loaded in memory.
     * @throws {DatabaseError} When database is not open.
     * @example
     * ```typescript
     * const collections = await db.listCollections();
     * console.log('Collections:', collections);
     * ```
     * @since 1.0.0
     */
    listCollections(): Promise<string[]>;
    /**
     * Check if a collection is currently loaded in memory.
     *
     * @param name - Name of the collection to check.
     * @returns True if collection exists in memory, false otherwise.
     * @example
     * ```typescript
     * if (db.hasCollection('users')) {
     *   const users = db.collection('users');
     * }
     * ```
     * @since 1.0.0
     */
    hasCollection(name: string): boolean;
    /**
     * Get aggregated statistics across all collections.
     *
     * @returns Database statistics including document counts, size, and uptime.
     * @throws {DatabaseError} When database is not open.
     * @example
     * ```typescript
     * const stats = await db.getStats();
     * console.log(`Database has ${stats.totalDocuments} documents across ${stats.collections} collections`);
     * ```
     * @since 1.0.0
     */
    getStats(): Promise<DatabaseStats>;
    /**
     * Get a copy of current database configuration.
     *
     * @returns Copy of the database options used during initialization.
     * @example
     * ```typescript
     * const options = db.getOptions();
     * console.log('Database path:', options.path);
     * ```
     * @since 1.0.0
     */
    getOptions(): DatabaseOptions;
    /**
     * Check if database is currently open and ready for operations.
     *
     * @returns True if database is open and ready, false otherwise.
     * @example
     * ```typescript
     * if (db.isDatabaseOpen()) {
     *   // Safe to perform operations
     * }
     * ```
     * @since 1.0.0
     */
    isDatabaseOpen(): boolean;
    /**
     * Get the storage path where database files are located.
     *
     * @returns Absolute or relative path to database storage directory.
     * @example
     * ```typescript
     * console.log('Database stored at:', db.getPath());
     * ```
     * @since 1.0.0
     */
    getPath(): string;
    /**
     * Clear all collection caches to free memory.
     *
     * @example
     * ```typescript
     * db.clearCaches(); // Free up memory by clearing all caches
     * ```
     * @since 1.0.0
     */
    clearCaches(): void;
    /**
     * Create a backup of the database (placeholder implementation).
     *
     * @param backupPath - Destination for backup files.
     */
    backup(backupPath: string): Promise<void>;
    /**
     * Perform database compaction to optimize storage (placeholder implementation).
     *
     * @throws {DatabaseError} When database is not open or compaction fails.
     * @example
     * ```typescript
     * await db.compact(); // Optimize storage space
     * ```
     * @since 1.0.0
     */
    compact(): Promise<void>;
    private setupLogger;
    private log;
    on<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    once<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    off<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
}

/**
 * Fluent query builder that provides a chainable API for building complex queries.
 * Similar to MongoDB's query builder but with a more JavaScript-friendly syntax.
 *
 * @typeParam T - Document type being queried.
 */
declare class QueryBuilder<T = Document> {
    private filter;
    private options;
    private collection;
    /**
     * Create a new query builder for a collection.
     *
     * @param collection - Collection instance to query.
     */
    constructor(collection: any);
    /**
     * Add a where clause to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator ($eq, $gt, etc.).
     * @param value    - Value to compare against.
     */
    where(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Add an AND condition to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator.
     * @param value    - Value to compare against.
     */
    and(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Add an OR condition to the query.
     *
     * @param field    - Document field to filter on.
     * @param operator - Comparison operator.
     * @param value    - Value to compare against.
     */
    or(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * Sort results by a field.
     *
     * @param field     - Field to sort by.
     * @param direction - 1 for ascending, -1 for descending.
     */
    sort(field: string, direction: 1 | -1): QueryBuilder<T>;
    /**
     * Limit the number of results returned.
     *
     * @param count - Maximum number of documents to return.
     */
    limit(count: number): QueryBuilder<T>;
    /**
     * Skip a number of results (for pagination).
     *
     * @param count - Number of documents to skip.
     */
    skip(count: number): QueryBuilder<T>;
    /**
     * Select specific fields to include in results.
     *
     * @param fields - Array of field names to include.
     */
    select(fields: string[]): QueryBuilder<T>;
    /**
     * Execute the query and return all matching results.
     *
     * @returns Query results containing documents, total count, and pagination info.
     * @throws {DatabaseError} When query execution fails.
     * @example
     * ```typescript
     * const results = await collection
     *   .query()
     *   .where('status', '$eq', 'active')
     *   .sort('name', 1)
     *   .limit(10)
     *   .execute();
     * console.log('Found:', results.documents.length);
     * ```
     * @since 1.0.0
     */
    execute(): Promise<FindResult<T>>;
    /**
     * Execute query and return only the first matching document.
     *
     * @returns First matching document or null if no matches found.
     * @throws {DatabaseError} When query execution fails.
     * @example
     * ```typescript
     * const user = await collection
     *   .query()
     *   .where('email', '$eq', 'user@example.com')
     *   .findOne();
     * ```
     * @since 1.0.0
     */
    findOne(): Promise<T | null>;
    /**
     * Count the number of documents that match the query without returning them.
     *
     * @returns Number of documents matching the current query filters.
     * @throws {DatabaseError} When count operation fails.
     * @example
     * ```typescript
     * const activeCount = await collection
     *   .query()
     *   .where('status', '$eq', 'active')
     *   .count();
     * ```
     * @since 1.0.0
     */
    count(): Promise<number>;
    /**
     * Check if any documents match the current query filters.
     *
     * @returns True if at least one document matches, false otherwise.
     * @throws {DatabaseError} When query execution fails.
     * @example
     * ```typescript
     * const hasActive = await collection
     *   .query()
     *   .where('status', '$eq', 'active')
     *   .exists();
     * ```
     * @since 1.0.0
     */
    exists(): Promise<boolean>;
    /**
     * Find the first matching document and update it atomically.
     *
     * @param updateData - Partial document data to apply to the matched document.
     * @returns Updated document or null if no matches found.
     * @throws {DatabaseError} When query or update operation fails.
     * @example
     * ```typescript
     * const updated = await collection
     *   .query()
     *   .where('_id', '$eq', 'user123')
     *   .findOneAndUpdate({ lastLogin: new Date() });
     * ```
     * @since 1.0.0
     */
    findOneAndUpdate(updateData: Partial<T>): Promise<T | null>;
    /**
     * Find the first matching document and delete it atomically.
     *
     * @returns Deleted document or null if no matches found.
     * @throws {DatabaseError} When query or delete operation fails.
     * @example
     * ```typescript
     * const deleted = await collection
     *   .query()
     *   .where('status', '$eq', 'expired')
     *   .findOneAndDelete();
     * if (deleted) {
     *   console.log('Deleted expired document:', deleted._id);
     * }
     * ```
     * @since 1.0.0
     */
    findOneAndDelete(): Promise<T | null>;
}

/**
 * Handles encryption and decryption of data using AES-256 with PBKDF2 key derivation.
 * Provides both string and object encryption utilities.
 */
declare class EncryptionManager {
    private key;
    private algorithm;
    /**
     * Initialize encryption manager with a key and algorithm.
     *
     * @param encryptionKey - Password/key for encryption.
     * @param algorithm     - Encryption algorithm (default: aes-256-cbc).
     */
    constructor(encryptionKey: string, algorithm?: string);
    /**
     * Derive encryption key from password using PBKDF2.
     *
     * @param password - Password to derive key from.
     * @param salt     - Optional salt (generates random if not provided).
     */
    private deriveKey;
    /**
     * Encrypt a string using AES-256-CBC.
     *
     * @param data - String to encrypt.
     * @returns Encrypted data in format "iv:encrypted".
     */
    encrypt(data: string): string;
    /**
     * Decrypt a string that was encrypted with this manager.
     *
     * @param encryptedData - Encrypted data in "iv:encrypted" format.
     * @returns Decrypted string.
     */
    decrypt(encryptedData: string): string;
    /**
     * Encrypt a JavaScript object by serializing to JSON first.
     *
     * @param obj - Object to encrypt.
     * @returns Encrypted JSON string.
     */
    encryptObject(obj: any): string;
    /**
     * Decrypt and deserialize a JavaScript object.
     *
     * @param encryptedData - Encrypted JSON string.
     * @returns Decrypted object.
     */
    decryptObject(encryptedData: string): any;
}

/**
 * Document with system metadata automatically added by NuboDB.
 */
type DocumentWithMetadata = Document & {
    _id: string;
    _createdAt: Date;
    _updatedAt: Date;
    _version?: number;
};
/**
 * Abstract base class that provides common functionality for collections.
 * Handles initialization, caching, indexing, and metadata management.
 *
 * @typeParam T - Document type for this collection.
 */
declare abstract class BaseCollection<T = Document> {
    protected name: string;
    protected storage: FileStorage;
    protected encryptionManager?: EncryptionManager;
    protected schema?: Schema;
    protected options: CollectionOptions;
    protected cache: Map<string, T>;
    protected indexes: Map<string, Map<any, string[]>>;
    protected isInitialized: boolean;
    protected documentCount: number;
    /**
     * Initialize the base collection with storage and options.
     *
     * @param name    - Collection name.
     * @param storage - Storage engine instance.
     * @param options - Collection configuration.
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /**
     * Load documents from storage, build indexes, and populate cache.
     * Called automatically before first operation.
     */
    initialize(): Promise<void>;
    /** Ensure collection is initialized before performing operations. */
    protected ensureInitialized(): Promise<void>;
    /** Get all documents from storage with decryption if enabled. */
    protected getAllDocuments(): Promise<T[]>;
    /**
     * Build in-memory indexes for fields marked as indexed in schema.
     *
     * @param documents - Documents to index.
     */
    protected buildIndexes(documents: T[]): Promise<void>;
    /**
     * Update indexes when documents are modified.
     *
     * @param document  - Document being modified.
     * @param operation - Type of operation (insert/update/delete).
     */
    protected updateIndexes(document: T & DocumentWithMetadata, operation: 'insert' | 'update' | 'delete'): Promise<void>;
    /**
     * Extract index key from document for compound indexes.
     *
     * @param document - Document to extract key from.
     * @param fields   - Fields to include in index key.
     */
    protected extractIndexKey(document: T & DocumentWithMetadata, fields: {
        [field: string]: 1 | -1;
    }): any;
    /** Clear the document cache to free memory. */
    clearCache(): void;
    /**
     * Get collection statistics.
     *
     * @returns Object with document count, size, index count, and cache size.
     */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
}

/**
 * Handles all document modification operations (CRUD) for a collection.
 * Extends BaseCollection to inherit caching, indexing, and initialization.
 *
 * @typeParam T - Document type for this collection.
 */
declare class DocumentOperations<T = Document> extends BaseCollection<T> {
    private queryOps;
    /**
     * Create document operations for a collection.
     *
     * @param name    - Collection name.
     * @param storage - Storage engine instance.
     * @param options - Collection configuration.
     */
    constructor(name: string, storage: any, options?: any);
    /**
     * Insert a single document with validation and metadata.
     *
     * @param data - Document data to insert.
     * @returns Insert result with generated ID and document.
     */
    insert(data: Partial<T>): Promise<InsertResult>;
    /**
     * Insert multiple documents in a single operation.
     *
     * @param documents - Array of documents to insert.
     * @returns Bulk insert result with IDs and count.
     */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /**
     * Update all documents matching a filter.
     *
     * @param filter     - Query filter to match documents.
     * @param updateData - Fields to update in matching documents.
     * @returns Update result with modified count.
     */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Update existing documents or insert new one if none match.
     *
     * @param filter     - Query filter to match documents.
     * @param updateData - Data to update or insert.
     * @returns Update result with upsert information.
     */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * Delete all documents matching a filter.
     *
     * @param filter - Query filter to match documents for deletion.
     * @returns Delete result with deleted count.
     */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Delete the first document that matches a filter.
     *
     * @param filter - Query filter to match document for deletion.
     * @returns Delete result with deleted count (0 or 1).
     */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * Manually create an index on specified fields.
     *
     * @param definition - Index definition with fields and options.
     */
    createIndex(definition: IndexDefinition): Promise<void>;
}

/**
 * Handles all query operations for a collection including filtering, sorting,
 * and pagination. Extends BaseCollection to inherit caching and indexing.
 *
 * @typeParam T - Document type for this collection.
 */
declare class QueryOperations<T = Document> extends BaseCollection<T> {
    /**
     * Find documents matching a filter with optional sorting and pagination.
     *
     * @param filter  - MongoDB-like query filter.
     * @param options - Query options (sort, limit, skip, projection).
     * @returns Query result with documents and metadata.
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /**
     * Find the first document matching a filter.
     *
     * @param filter - Query filter to match documents.
     * @returns First matching document or null.
     */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /**
     * Find a document by its unique ID (with cache lookup).
     *
     * @param id - Document ID to lookup.
     * @returns Document or null if not found.
     */
    findById(id: string): Promise<T | null>;
    /** Create a new QueryBuilder instance for this collection. */
    query(): QueryBuilder$1<T>;
    /**
     * Count documents matching a filter.
     *
     * @param filter - Query filter to match documents.
     * @returns Number of matching documents.
     */
    count(filter?: QueryFilter): Promise<number>;
    /** Check if collection has no documents. */
    isEmpty(): Promise<boolean>;
    /**
     * Apply filter to a list of documents.
     *
     * @param documents - Documents to filter.
     * @param filter    - Query filter to apply.
     */
    private filterDocumentsOptimized;
    /**
     * Check if a document matches a query filter.
     *
     * @param document - Document to check.
     * @param filter   - Query filter to match against.
     */
    private matchesFilterOptimized;
    /**
     * Handle logical operators ($and, $or, $nor) in query filters.
     *
     * @param document  - Document to check.
     * @param operator  - Logical operator.
     * @param conditions - Array of conditions to evaluate.
     */
    private matchesLogicalOperator;
    /**
     * Handle comparison operators ($eq, $gt, $lt, etc.) in query filters.
     *
     * @param value     - Document field value.
     * @param operators - Comparison operators to apply.
     */
    private matchesComparisonOperatorsOptimized;
    /**
     * Sort documents by specified fields.
     *
     * @param documents - Documents to sort.
     * @param sort      - Sort specification (field: direction).
     */
    private sortDocumentsOptimized;
    /**
     * Apply field projection to documents (include/exclude fields).
     *
     * @param documents  - Documents to project.
     * @param projection - Field projection specification.
     */
    private projectDocumentsOptimized;
    /**
     * Use available indexes to optimize document filtering.
     *
     * @param documents - Documents to filter.
     * @param filter    - Query filter to apply.
     */
    private useIndexesOptimized;
    private filterDocuments;
    private matchesFilter;
    private matchesComparisonOperators;
    private sortDocuments;
    private projectDocuments;
    private useIndexes;
}

/**
 * Base error class for all database-related errors.
 */
declare class DatabaseError extends Error {
    readonly code: string;
    readonly details?: any;
    /**
     * Create a new database error.
     *
     * @param message - Error message.
     * @param code    - Error code for programmatic handling.
     */
    constructor(message: string, code: string, details?: any);
}
/**
 * Error thrown when collection operations fail.
 */
declare class CollectionError extends DatabaseError {
    constructor(message: string, details?: any);
}
/**
 * Error thrown when document operations fail.
 */
declare class DocumentError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class SchemaError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class EncryptionError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class StorageError extends DatabaseError {
    constructor(message: string, details?: any);
}

/**
 * Generate a unique document ID using crypto.randomUUID().
 *
 * @returns A unique string identifier.
 */
declare function generateId(): string;
/**
 * Get the current timestamp as a Date object.
 *
 * @returns Current date and time.
 */
declare function generateTimestamp(): Date;
/**
 * Create metadata for a new document.
 *
 * @returns Object with ID and timestamps.
 */
declare function createDocumentMetadata(id?: string): {
    id: string;
    createdAt: Date;
    updatedAt: Date;
};

/**
 * Validate document data against a schema definition.
 *
 * @param data   - Document data to validate.
 * @param schema - Schema definition to validate against.
 * @throws Error if validation fails.
 */
declare function validateSchema(data: any, schema: Schema): void;
/**
 * Apply default values from schema to document data.
 *
 * @param data   - Document data.
 * @param schema - Schema with default values.
 * @returns Data with defaults applied.
 */
declare function applyDefaults(data: any, schema: Schema): any;
/**
 * Remove undefined values and sanitize document data.
 *
 * @param data - Raw document data.
 * @returns Sanitized document data.
 */
declare function sanitizeDocument(data: any): Omit<Document, '_id' | '_createdAt' | '_updatedAt'>;

/**
 * Convenience function for creating database instances.
 *
 * @param options - Database configuration options.
 * @returns Promise resolving to a database instance.
 */
declare function createDatabase(options: DatabaseOptions): Promise<NuboDB>;
declare const getDatabase: typeof NuboDB.getInstance;

export { BaseCollection, Collection, CollectionError, type CollectionOptions, DatabaseError, type DatabaseEvents, type DatabaseStats, type DeleteResult, type Document, DocumentError, type DocumentMetadata, DocumentOperations, type DocumentWithMetadata, EncryptionError, EncryptionManager, FileStorage, type FindResult, type IndexDefinition, type InsertManyResult, type InsertResult, NuboDB, QueryBuilder, type QueryBuilder$1 as QueryBuilderType, type QueryCondition, type QueryFilter, QueryOperations, type QueryOperator, type QueryOptions, type Schema, SchemaError, type SchemaField, StorageError, type Transaction, type UpdateResult, applyDefaults, createDatabase, createDocumentMetadata, NuboDB as default, generateId, generateTimestamp, getDatabase, sanitizeDocument, validateSchema };
